using System;
using System.Text;
using Microsoft.SPOT;
using System.Threading;


/// <summary>
///  Command() is the base class for any of the Cmd... classes. It contains the basic functions
///  that are available to the derived classes as well.
/// </summary>
    class Command
    {
        // class declarations
        public enum parameterType { INTEGER, STRING };
        public enum serverFlags { NONE=0, LOGGED_IN=1, DISCONNECT_REQUEST=2, NOP_RECEIVED=3};
        private Velleman_K8000 _k8000;
        public Parameter[] parameterList = new Parameter[0];
        private string _keyword = "";
        private int _parameters = 0;
        private string _error = "";
        private static bool _isBusy = false;
        private string _response = "";
        private serverFlags _serverFlag = serverFlags.NONE;



        #region "properties"
        /// <summary>
        /// The K8000 property allows you to get access to the K8000 card.
        /// </summary>
        public Velleman_K8000 K8000
        {
            get { return _k8000; }
            set
            {
                _k8000 = value;
            }
        }


        /// <summary>
        /// The serverflag property allows you to set the connection's properties.
        /// It is used only for a few cases.
        /// </summary>
        public serverFlags serverflag
        {
            get { return _serverFlag; }
            set
            {
                _serverFlag = value;
            }
        }


        /// <summary>
        /// response contains the String that returned while executing the K8000 command
        /// </summary>
        public string response
        {
            get { return _response; }
            set
            {
                _response = value;
            }
        }


        /// <summary>
        /// keyword contains the instruction that is understood by the server.
        /// </summary>
        public string keyword
        {
            get { return _keyword; }
            set
            {
                _keyword = value;
            }
        }


        /// <summary>
        /// parameters contains the number of parameters
        /// </summary>
        public int parameters
        {
            get { return _parameters; }
            set
            {
                _parameters = value;
            }
        }


        /// <summary>
        /// error contains the error that last occured.
        /// </summary>
        public string error
        {
            get { return _error; }
            set
            {
                _error = value;
            }
        }


        /// <summary>
        /// isBusy is used to detect if there is already a command running.
        /// </summary>
        public static bool isBusy
        {
            get { return _isBusy; }
            set
            {
                _isBusy = value;
            }
        }

        #endregion



        /// <summary>
        /// executeCommand() will execute the received. You need to pass the array with all words that were
        /// received by the server.
        /// 
        /// The function will first verify if the parameters correspond with the expected parameters: 
        /// 1) is the parameter count as expected?
        /// 2) is the type of each parameter right?
        /// 3) is the value of the integer parameters within limits?
        /// 
        /// </summary>
        /// <param name="segments">An array with all elements</param>
        /// <returns>A boolean to indicate if the function succeeded</returns>
        public bool executeCommand(string[] segments)
        {
            error = "";
            response = "";

            // first compare the number of parameters
            if (segments.Length - 1 == parameters)
            {   // parameter count match. Now we need to verify the parameters one by one
                for (int i = 0; i < segments.Length-1; i++)
                {
                    bool result = parameterList[i].assignIfPossible(segments[i + 1]);

                    if (!result)
                    {
                        error = "command has one or more invalid parameters";
                        return false; 
                    }
                }

                try
                {
                    // here we will start the actual calls on the K8000 card.
                    if (!Command.isBusy)
                    {
                        Command.isBusy = true;
                        ThreadPriority prevPriority = Thread.CurrentThread.Priority;
                        Thread.CurrentThread.Priority = ThreadPriority.AboveNormal;
                        response = executeCommand();
                        Thread.CurrentThread.Priority = prevPriority;
                        Command.isBusy = false; 
                        return true;
                    }
                    else
                    {
                        error = "K8000 busy";
                        return false; 
                    }
                }
                catch (Exception e)
                {
                    Command.isBusy = false;
                    error = "K8000 execution error!";
                    error = e.Message;  // todo: make sure it is never multiline!
                    Debug.Print("error while executing K8000 command: " + e.Message); 
                    return false; 
                }        
            }
            else
            {   // parameter count mismatch. Do nothing!
                error = "wrong number of parameters";
                return false;
            }

            
        }



        /// <summary>
        /// executeCommand() will execute the command on the K8000 card. In fact this function will
        /// not be executed. Instead, the function inside the derived Cmd... class is executed.
        /// </summary>
        /// <returns>The string with the result from the execution</returns>
        public virtual String executeCommand() {
            return "this message should never occur!";
        }



        #region "Parameter Management"
        /// <summary>
        /// addParameter() adds a new parameter to the list with parameters. This function should be called
        /// from the constructor of the Cmd... classes.
        /// </summary>
        /// <param name="newParameter">newParameter</param>
        private void addParameter(Parameter newParameter)
        {
            int arraySize = parameterList.Length;
            Parameter[] tempArray = new Parameter[arraySize + 1];
            if (parameterList != null)
            {
                System.Array.Copy(parameterList, tempArray, System.Math.Min(parameterList.Length, tempArray.Length));
            }
            parameterList = tempArray;
            parameterList[arraySize] = newParameter;
        }



        /// <summary>
        /// getParameterByName() returns a parameter object from the parameter list. The
        /// parameter is found by its name. This is why a parameter name should be unique.
        /// </summary>
        /// <param name="paramName">The name of the parameter to find</param>
        /// <returns>The parameter object</returns>
        public Parameter getParameterByName(string paramName)
        {
            for (int i = 0; i < parameterList.Length; i++)
            {
                if (parameterList[i].name == paramName)
                {   // we found the object
                    return parameterList[i];
                }
            }

            return null;
        }



        /// <summary>
        /// getIntParameter() receives an Integer value from the parameter list.
        /// </summary>
        /// <param name="paramName">The parameter to receive</param>
        /// <returns>An Integer containing the requested value</returns>
        public int getIntParameter(string paramName) 
        {
            Parameter param = getParameterByName(paramName);
            if (param != null)
            {
                return param.intValue;
            }
            else
                return -1;  // should never occur if programming is right
        }




        /// <summary>
        /// getStringParameter() receives a String value from the parameter list.
        /// </summary>
        /// <param name="paramName">The parameter to receive</param>
        /// <returns>A string containing the requested value</returns>
        public string getStringParameter(string paramName)
        {
            Parameter param = getParameterByName(paramName);
            if (param != null)
            {
                return param.stringValue;
            }
            else
                return "";  // should never occur if programming is right
        }




        /// <summary>
        /// addParameter adds a parameter to the class.
        /// </summary>
        /// <param name="name">The name for the parameter. It should be unique</param>
        /// <param name="typeOfParameter">The type of the parameter (see enum PARAMETERTYPE)</param>
        /// <param name="maxLength">The length of the parameter</param>
        public void addParameter(string name, parameterType typeOfParameter, int maxLength)
        {
            Parameter param = new Parameter(name, typeOfParameter, maxLength);
            addParameter(param);
            parameters++;
        }




        /// <summary>
        /// addParameter adds a parameter to the class.
        /// </summary>
        /// <param name="name">The name for the parameter. It should be unique</param>
        /// <param name="typeOfParameter">The type of the parameter (see enum PARAMETERTYPE)</param>
        /// <param name="min">The min value of the variable</param>
        /// <param name="max">The max value of the variable</param>
        public void addParameter(string name, parameterType typeOfParameter, int min, int max)
        {
            Parameter param = new Parameter(name, typeOfParameter, min,max);
            addParameter(param);
            parameters++;
        }
        #endregion
    }

